'use strict';

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _path = require('path');

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _videoExtensions = require('video-extensions');

var _videoExtensions2 = _interopRequireDefault(_videoExtensions);

var _movieTitle = require('movie-title');

var _movieTitle2 = _interopRequireDefault(_movieTitle);

var _omdb = require('omdb');

var _omdb2 = _interopRequireDefault(_omdb);

var _throat = require('throat');

var _throat2 = _interopRequireDefault(_throat);

var _pify = require('pify');

var _pify2 = _interopRequireDefault(_pify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = (0, _debug2.default)('movieList');

const DEFAULT_GLOB = [`**/*.{${ _videoExtensions2.default.join(',') }}`, '!**/*{sample,Sample,rarbg.com,RARBG.com}*.*'];
const DEFAULT_CONCURRENT_REQUESTS = 15;

module.exports = function () {
  var _ref = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var _ref$movieGlob = _ref.movieGlob;
  let movieGlob = _ref$movieGlob === undefined ? DEFAULT_GLOB : _ref$movieGlob;
  var _ref$source = _ref.source;
  let source = _ref$source === undefined ? process.cwd() : _ref$source;
  var _ref$concurrentReques = _ref.concurrentRequests;
  let concurrentRequests = _ref$concurrentReques === undefined ? DEFAULT_CONCURRENT_REQUESTS : _ref$concurrentReques;

  debug('resolved options: %j', {
    movieGlob: movieGlob,
    source: source,
    concurrentRequests: concurrentRequests
  });

  if (typeof concurrentRequests !== 'number') {
    throw new TypeError('concurrentRequests must be a number');
  }

  if (typeof source !== 'string') {
    throw new TypeError('source must be a string');
  }

  if (typeof movieGlob !== 'string' && !Array.isArray(movieGlob)) {
    throw new TypeError('movieGlob must be a string or a string array');
  }

  return (0, _globby2.default)(movieGlob, { cwd: source }).then(files => {
    debug('files found: %d', files.length);
    const getMovieData = file => {
      const movieInfo = {
        path: file,
        name: (0, _movieTitle2.default)((0, _path.basename)(file))
      };

      return (0, _pify2.default)(_omdb2.default.get)(movieInfo.name).then(info => {
        movieInfo.info = info;

        if (!info) {
          const error = new Error('Movie not found');

          error.code = 'ENOMOVIE';

          return {
            state: 'failed',
            value: movieInfo,
            reason: error
          };
        }

        return {
          state: 'succeeded',
          value: movieInfo
        };
      }, err => {
        return {
          state: 'failed',
          value: movieInfo,
          reason: err
        };
      });
    };

    return Promise.all(files.map((0, _throat2.default)(concurrentRequests, getMovieData)));
  }).then(results => {
    debug('results: %j', results);

    return (0, _lodash2.default)(results).groupBy('state').mapValues((value, key) => key === 'succeeded' ? _lodash2.default.map(value, 'value') : value).value();
  });
};
//# sourceMappingURL=index.js.map